;; -*- mode: Lisp; Syntax: Common-Lisp; Package: cells; -*-
#|

    Hunchncells -- Cells, Parenscript, and HTML

Copyright (C) 2008 by Andy Chambers

This library is free software; you can redistribute it and/or
modify it under the terms of the Lisp Lesser GNU Public License
 (http://opensource.franz.com/preamble.html), known as the LLGPL.

This library is distributed  WITHOUT ANY WARRANTY; without even 
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the Lisp Lesser GNU Public License for more details.

|#

(in-package :hunchncells)


;; (defmd ogl-node ()
;;   (dsp-list
;;      (c-formula (:lazy :until-asked)
;;         (without-c-dependency
;;             (map nil 'dsp-list (kids self)))
;;         (let ((display-list-name (or .cache (gl-gen-lists 1)))) ; #1
;;             ;---start building this list -----
;;             (gl-new-list display-list-name gl_compile)
;;             (ix-paint self)
;;             (gl-end-list)
;;             ;---end building
;;             (setf (redisplayp .og.) t) ; Note #2
;;             display-list-name))) ;; always the same once allocated

;; #1 display-list-name (actually just an integer) is like an OpenGL object
;; ID. When larger structures render, they render kids by "calling" the
;; kids' display lists by these names. gl-call-list or something.

;; #2 notice that this rule mutates the display list, it does not allocate
;; a new one. (The whole scheme breaks down if we do that.) So we will
;; forever be returning the same "name" and so the Cells engine will not
;; see any change. Observers will not run and (crucially) users will not
;; get invalidated. This is way cool because this is exactly what we want.
;; The only dependencies arise during that call you see to IX-PAINT. If I
;; reference any other cell and that changes, then when the name slot gets
;; sampled Cells will kick off the rule to rebuild the list.



(defmodel html (family)
  ((id :initarg :id :accessor id :initform nil)
   (class :initarg :class :accessor cls :initform nil)
   (title :initarg :title :accessor title :initform nil)
   (style :initarg :style :accessor style :initform nil)
   (xuid :initarg :xuid :accessor xuid :initform nil)
   (dirtyp :initarg :dirtyp :accessor dirtyp :initform nil :cell nil)
   (xhtml :initarg :xhtml :accessor xhtml :initform nil)))







;;; Grouping
(defhtml div () ())
(defhtml span () ())

;;; Headers
(defhtml h1 () ())
(defhtml h2 () ())
(defhtml h3 () ())

;;; Text

(defmodel text (html)
  ()
  (:default-initargs
      :xhtml (c? (with-output-to-string (s)
		   (with-html-output (s nil :prologue nil :indent t)
		     (:span :id (^id)
			    (str (^value))))))))

(defmacro mk-text (text)
  `(make-instance
    'text
    :fm-parent *parent*
    :id (gensym "S")
    :value ,text))


  


;; (xhtml (mk-p (:id "some p")
;; 	 (mk-em ()
;; 	   (mk-strong ()
;; 	     (mk-text "fdfd")))
;; 	 (mk-text "fdf")))

;; why have a special model for text you ask??!!
;; three reasons
;;  a) fits in cleanly to the family tree
;;  b) we can use the xhtml rule to escape special
;;     characters
;;  c) we put all text inside addressable span elements
;;     that we'll use for dynamic updates


(defmodel page (html)
  ()
  (:default-initargs
      :xhtml (c? (with-output-to-string (s)
		   (with-html-output (s nil :prologue t :indent t)
		     (:html
		      (:head
		       (:title (str (^title)))
		       (:style :type "text/css"
			       :href (^style)))
		      (:body
		       (:div :id "banner"
			     (:h1 (str (^title))))
		       (:div :id "content"
			     (str (apply #'concatenate 'string
					 (mapcar 'xhtml (^kids))))))))))))


;; (defun test-mk ()
;;   (xhtml
;;    (mk-page ()
;;      (mk-div ()))))

;; (xhtml )
	 
;; 	   (mk-div (:id "aa"))))
