;; -*- mode: Lisp; Syntax: Common-Lisp; Package: cells; -*-
#|

    Hunchncells -- Cells, Parenscript, and HTML

Copyright (C) 2008 by Andy Chambers

This library is free software; you can redistribute it and/or
modify it under the terms of the Lisp Lesser GNU Public License
 (http://opensource.franz.com/preamble.html), known as the LLGPL.

This library is distributed  WITHOUT ANY WARRANTY; without even 
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the Lisp Lesser GNU Public License for more details.

|#

(in-package :hunchncells)

;; Helpers

(defun defslot (name)
  `(,name :initform nil
	  :initarg ,(intern (string name) :keyword)))

(defun xmlrule (class html-attrs)
  `(c? (with-output-to-string (s)
;; 	 (without-c-dependency
;; 	   (map nil #'xhtml (^kids)))

	 (with-html-output (s nil :prologue nil :indent t)
	   (,(intern (symbol-name class) :keyword)
	   ,@(loop for (cl-meth attr) in html-attrs
		append `(,(intern (symbol-name attr) :keyword)
			  (,(intern (format nil "^~a" (symbol-name cl-meth))))))
	   (str (apply #'concatenate 'string
		       (mapcar 'xhtml (^kids)))))))))

(defun options-normalize (options)
  (loop for option in options
      for slot-name = (intern (string (if (listp option)
					  (car option)
					  option)))
      collecting (list slot-name (if (listp option)
				     (cadr option)
				     option))))

(defun define-model (class superclasses std-slots html-attrs defclass-options)
  `(defmodel ,class ,(or superclasses '(html))
     (,@(append std-slots
		(loop for (slot-name nil) in html-attrs
		     collecting (defslot (intern (symbol-name slot-name))))))
     ,@(remove-if (lambda (k)
		    (find k '(:default-initargs :attrs)))
		  defclass-options
		  :key 'car)
     (:default-initargs
	 ,@(cdr (find :default-initargs defclass-options :key 'car))
	 :xhtml ,(xmlrule class html-attrs)
	 :xuid (c-formula (:lazy :until-asked)
		     (without-c-dependency
		       (map nil 'xuid (kids self)))
		     (let ((xuid (or .cache
				     (or (^id)
					 (gensym)))))
		       (push self (updates (fm-top self)))
		       xuid)))))

(defun define-mk (class)
  `(defmacro ,(intern (concatenate
		       'string "MK-" (symbol-name class)))
       ((&rest inits) &body body)
     `(make-instance
       ',',class
       ,@inits
       :fm-parent *parent*
       :id (gensym (subseq (string ',',class) 0 1))
       :kids (c? (the-kids
		  ,@(loop for kid in body
                      when (stringp kid)
		       collect `(mk-text ,kid)
		      else
		       collect kid))))))


(defmacro defhtml (class superclasses
		   (&rest std-slots)
		   &body defclass-options)
  (destructuring-bind (&optional html-class &rest html-attrs)
      (cdr (find :attrs defclass-options :key 'car))
    (declare (ignore html-class))

    (setf html-attrs (options-normalize html-attrs))

    `(progn
       ,(define-model class superclasses std-slots html-attrs defclass-options)

       ,(define-mk class)

       (find-class ',class))))




(defmacro mk-page ((&rest inits) &body body)
  `(make-instance 
    'page
    ,@inits
    :fm-parent *parent*
    :kids (c? (the-kids 
	       ,@body))))


(defmacro mk-div ((&rest inits) &body body)
  `(make-instance
    'div
    ,@inits
    :fm-parent *parent*
    :kids (c? (the-kids
	       ,@(loop for kid in body
		      when (stringp kid)
		       collect `(mk-text ,kid)
		      else
		       collect kid)))))

